@@ -77,7 +77,7 @@ function wp_default_scripts( &$scripts ) {
 
 	$scripts->add( 'common', "/wp-admin/js/common$suffix.js", array('jquery', 'hoverIntent', 'utils'), false, 1 );
 	did_action( 'init' ) && $scripts->localize( 'common', 'commonL10n', array(
-		'warnDelete'   => __( "You are about to permanently delete these items.\n  'Cancel' to stop, 'OK' to delete." ),
+		'warnDelete'   => __( "You are about to permanently delete these items from your site.\nThis action cannot be undone.\n 'Cancel' to stop, 'OK' to delete." ),
 		'dismiss'      => __( 'Dismiss this notice.' ),
 		'collapseMenu' => __( 'Collapse Main menu' ),
 		'expandMenu'   => __( 'Expand Main menu' ),
@@ -280,7 +280,7 @@ function wp_default_scripts( &$scripts ) {
 		'queue_limit_exceeded' => __('You have attempted to queue too many files.'),
 		'file_exceeds_size_limit' => __('%s exceeds the maximum upload size for this site.'),
 		'zero_byte_file' => __('This file is empty. Please try another.'),
-		'invalid_filetype' => __('This file type is not allowed. Please try another.'),
+		'invalid_filetype' => __('Sorry, this file type is not permitted for security reasons.'),
 		'not_an_image' => __('This file is not an image. Please try another.'),
 		'image_memory_exceeded' => __('Memory exceeded. Please try another smaller file.'),
 		'image_dimensions_exceeded' => __('This is larger than the maximum size. Please try another.'),
@@ -357,7 +357,7 @@ function wp_default_scripts( &$scripts ) {
 			'Play'                    => __( 'Play' ),
 			'Pause'                   => __( 'Pause' ),
 			'Captions/Subtitles'      => __( 'Captions/Subtitles' ),
-			'None'                    => __( 'None', 'no captions/subtitles' ),
+			'None'                    => _x( 'None', 'no captions/subtitles' ),
 			'Time Slider'             => __( 'Time Slider' ),
 			/* translators: %1: number of seconds (30 by default) */
 			'Skip back %1 seconds'    => __( 'Skip back %1 seconds' ),
@@ -602,6 +602,14 @@ function wp_default_scripts( &$scripts ) {
 		$scripts->add( 'admin-gallery', "/wp-admin/js/gallery$suffix.js", array( 'jquery-ui-sortable' ) );
 
 		$scripts->add( 'admin-widgets', "/wp-admin/js/widgets$suffix.js", array( 'jquery-ui-sortable', 'jquery-ui-draggable', 'jquery-ui-droppable' ), false, 1 );
+		$scripts->add( 'media-widgets', "/wp-admin/js/widgets/media-widgets$suffix.js", array( 'jquery', 'media-models', 'media-views' ) );
+		$scripts->add_inline_script( 'media-widgets', 'wp.mediaWidgets.init();', 'after' );
+
+		$scripts->add( 'media-audio-widget', "/wp-admin/js/widgets/media-audio-widget$suffix.js", array( 'media-widgets', 'media-audiovideo' ) );
+		$scripts->add( 'media-image-widget', "/wp-admin/js/widgets/media-image-widget$suffix.js", array( 'media-widgets' ) );
+		$scripts->add( 'media-video-widget', "/wp-admin/js/widgets/media-video-widget$suffix.js", array( 'media-widgets', 'media-audiovideo' ) );
+		$scripts->add( 'text-widgets', "/wp-admin/js/widgets/text-widgets$suffix.js", array( 'jquery', 'backbone', 'editor', 'wp-util', 'wp-a11y' ) );
+		$scripts->add_inline_script( 'text-widgets', 'wp.textWidgets.init();', 'after' );
 
 		$scripts->add( 'theme', "/wp-admin/js/theme$suffix.js", array( 'wp-backbone', 'wp-a11y' ), false, 1 );
 
@@ -637,32 +645,34 @@ function wp_default_scripts( &$scripts ) {
 				'noPlugins'                  => __( 'You do not appear to have any plugins available at this time.' ),
 				'noItemsSelected'            => __( 'Please select at least one item to perform this action on.' ),
 				'updating'                   => __( 'Updating...' ), // No ellipsis.
-				'updated'                    => __( 'Updated!' ),
+				'pluginUpdated'              => _x( 'Updated!', 'plugin' ),
+				'themeUpdated'               => _x( 'Updated!', 'theme' ),
 				'update'                     => __( 'Update' ),
 				'updateNow'                  => __( 'Update Now' ),
 				/* translators: %s: Plugin name and version */
-				'updateNowLabel'             => __( 'Update %s now' ),
+				'pluginUpdateNowLabel'       => _x( 'Update %s now', 'plugin' ),
 				'updateFailedShort'          => __( 'Update Failed!' ),
 				/* translators: %s: Error string for a failed update */
 				'updateFailed'               => __( 'Update Failed: %s' ),
 				/* translators: %s: Plugin name and version */
-				'updatingLabel'              => __( 'Updating %s...' ), // No ellipsis.
+				'pluginUpdatingLabel'        => _x( 'Updating %s...', 'plugin' ), // No ellipsis.
 				/* translators: %s: Plugin name and version */
-				'updatedLabel'               => __( '%s updated!' ),
+				'pluginUpdatedLabel'         => _x( '%s updated!', 'plugin' ),
 				/* translators: %s: Plugin name and version */
-				'updateFailedLabel'          => __( '%s update failed' ),
-				/* translators: JavaScript accessible string */
+				'pluginUpdateFailedLabel'    => _x( '%s update failed', 'plugin' ),
+				/* translators: Accessibility text */
 				'updatingMsg'                => __( 'Updating... please wait.' ), // No ellipsis.
-				/* translators: JavaScript accessible string */
+				/* translators: Accessibility text */
 				'updatedMsg'                 => __( 'Update completed successfully.' ),
-				/* translators: JavaScript accessible string */
+				/* translators: Accessibility text */
 				'updateCancel'               => __( 'Update canceled.' ),
 				'beforeunload'               => __( 'Updates may not complete if you navigate away from this page.' ),
 				'installNow'                 => __( 'Install Now' ),
 				/* translators: %s: Plugin name */
-				'installNowLabel'            => __( 'Install %s now' ),
+				'pluginInstallNowLabel'      => _x( 'Install %s now', 'plugin' ),
 				'installing'                 => __( 'Installing...' ),
-				'installed'                  => __( 'Installed!' ),
+				'pluginInstalled'            => _x( 'Installed!', 'plugin' ),
+				'themeInstalled'             => _x( 'Installed!', 'theme' ),
 				'installFailedShort'         => __( 'Install Failed!' ),
 				/* translators: %s: Error string for a failed installation */
 				'installFailed'              => __( 'Installation failed: %s' ),
@@ -691,7 +701,8 @@ function wp_default_scripts( &$scripts ) {
 				'deleting'                   => __( 'Deleting...' ),
 				/* translators: %s: Error string for a failed deletion */
 				'deleteFailed'               => __( 'Deletion failed: %s' ),
-				'deleted'                    => __( 'Deleted!' ),
+				'pluginDeleted'              => _x( 'Deleted!', 'plugin' ),
+				'themeDeleted'               => _x( 'Deleted!', 'theme' ),
 				'livePreview'                => __( 'Live Preview' ),
 				'activatePlugin'             => is_network_admin() ? __( 'Network Activate' ) : __( 'Activate' ),
 				'activateTheme'              => is_network_admin() ? __( 'Network Enable' ) : __( 'Activate' ),
@@ -721,7 +732,7 @@ function wp_default_scripts( &$scripts ) {
 			'current' => __( 'Current Color' ),
 		) );
 
-		$scripts->add( 'dashboard', "/wp-admin/js/dashboard$suffix.js", array( 'jquery', 'admin-comments', 'postbox' ), false, 1 );
+		$scripts->add( 'dashboard', "/wp-admin/js/dashboard$suffix.js", array( 'jquery', 'admin-comments', 'postbox', 'wp-util', 'wp-a11y' ), false, 1 );
 
 		$scripts->add( 'list-revisions', "/wp-includes/js/wp-list-revisions$suffix.js" );
 
@@ -834,7 +845,7 @@ function wp_default_styles( &$styles ) {
 	$styles->add( 'themes',              "/wp-admin/css/themes$suffix.css" );
 	$styles->add( 'about',               "/wp-admin/css/about$suffix.css" );
 	$styles->add( 'nav-menus',           "/wp-admin/css/nav-menus$suffix.css" );
-	$styles->add( 'widgets',             "/wp-admin/css/widgets$suffix.css" );
+	$styles->add( 'widgets',             "/wp-admin/css/widgets$suffix.css", array( 'wp-pointer' ) );
 	$styles->add( 'site-icon',           "/wp-admin/css/site-icon$suffix.css" );
 	$styles->add( 'l10n',                "/wp-admin/css/l10n$suffix.css" );
 
@@ -989,6 +1000,70 @@ function wp_localize_jquery_ui_datepicker() {
 	wp_add_inline_script( 'jquery-ui-datepicker', "jQuery(document).ready(function(jQuery){jQuery.datepicker.setDefaults({$datepicker_defaults});});" );
 }
 
+/**
+ * Localizes community events data that needs to be passed to dashboard.js.
+ *
+ * @since 4.8.0
+ */
+function wp_localize_community_events() {
+	if ( ! wp_script_is( 'dashboard' ) ) {
+		return;
+	}
+
+	require_once( ABSPATH . 'wp-admin/includes/class-wp-community-events.php' );
+
+	$user_id            = get_current_user_id();
+	$saved_location     = get_user_option( 'community-events-location', $user_id );
+	$saved_ip_address   = isset( $saved_location['ip'] ) ? $saved_location['ip'] : false;
+	$current_ip_address = WP_Community_Events::get_unsafe_client_ip();
+
+	/*
+	 * If the user's location is based on their IP address, then update their
+	 * location when their IP address changes. This allows them to see events
+	 * in their current city when travelling. Otherwise, they would always be
+	 * shown events in the city where they were when they first loaded the
+	 * Dashboard, which could have been months or years ago.
+	 */
+	if ( $saved_ip_address && $current_ip_address && $current_ip_address !== $saved_ip_address ) {
+		$saved_location['ip'] = $current_ip_address;
+		update_user_option( $user_id, 'community-events-location', $saved_location, true );
+	}
+
+	$events_client = new WP_Community_Events( $user_id, $saved_location );
+
+	wp_localize_script( 'dashboard', 'communityEventsData', array(
+		'nonce' => wp_create_nonce( 'community_events' ),
+		'cache' => $events_client->get_cached_events(),
+
+		'l10n' => array(
+			'enter_closest_city' => __( 'Enter your closest city to find nearby events.' ),
+			'error_occurred_please_try_again' => __( 'An error occurred. Please try again.' ),
+			'attend_event_near_generic' => __( 'Attend an upcoming event near you.' ),
+
+			/*
+			 * These specific examples were chosen to highlight the fact that a
+			 * state is not needed, even for cities whose name is not unique.
+			 * It would be too cumbersome to include that in the instructions
+			 * to the user, so it's left as an implication.
+			 */
+			/* translators: %s is the name of the city we couldn't locate.
+			 * Replace the examples with cities related to your locale. Test that
+			 * they match the expected location and have upcoming events before
+			 * including them. If no cities related to your locale have events,
+			 * then use cities related to your locale that would be recognizable
+			 * to most users. Use only the city name itself, without any region
+			 * or country. Use the endonym (native locale name) instead of the
+			 * English name if possible.
+			 */
+			'could_not_locate_city' => __( 'We couldn&#8217;t locate %s. Please try another nearby city. For example: Kansas City; Springfield; Portland.' ),
+
+			// This one is only used with wp.a11y.speak(), so it can/should be more brief.
+			/* translators: %s is the name of a city. */
+			'city_updated' => __( 'City updated. Listing events near %s.' ),
+		)
+	) );
+}
+
 /**
  * Administration Screen CSS for changing the styles.
  *
